package djmil.cordacheckers.contracts;

import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleCommand;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleInputSar;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleReferenceSar;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.Command;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameBoardContract implements net.corda.v5.ledger.utxo.Contract {

    private final static Logger log = LoggerFactory.getLogger(GameBoardContract.class);

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameBoardContract.verify() called");

        requireThat(trx.getCommands().size() == 1, REQUIRE_SINGLE_COMMAND);
        final Command command = getSingleCommand(trx, Command.class);

        if (command instanceof GameProposalCommand) {
            log.info("GameBoardContract.verify() as GameProposalCommand "+(GameProposalCommand)command);
            switch ((GameProposalCommand)command) {
                case ACCEPT:
                    GameProposalCommand.validateAcceptTrx(trx);
                    break;

                default:
                    throw new CordaRuntimeException("UNKNOWN_COMMAND");
            }
        } else 
        if (command instanceof GameBoardCommand) {
            log.info("GameBoardContract.verify() as GameBoardCommand "+((GameBoardCommand)command).getType());
            switch (((GameBoardCommand)command).getType()) {
                case MOVE:
                break;
                
                case SURRENDER:
                    GameBoardCommand.validateSurrenderTrx(trx);
                    break;

                case DRAW:
                break;
                case VICTORY:
                break;
                case ACCEPT:
                break;
                default:
                    throw new GameBoardCommand.CommandTypeException();
            }
        } else {
            throw new RuntimeException("Bad utxo command type");
        }
    }

    
    private static void requireThat(boolean asserted, String errorMessage) {
        if(!asserted) {
            throw new CordaRuntimeException("Failed requirement: " + errorMessage);
        }
    }

    static final String REQUIRE_SINGLE_COMMAND = "Require a single command";

    static final String SINGLE_STATE_EXPECTED = "Single state expected";
    static final String NO_REFERANCE_GAMEPROPOSAL_STATE_FOR_TRXID = "No reference GamePropsal state found for trx.id ";
}
