package djmil.cordacheckers.contracts;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleCommand;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.requireThat;

public class GameProposalContract implements net.corda.v5.ledger.utxo.Contract {

    private final static Logger log = LoggerFactory.getLogger(GameProposalContract.class);

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameProposalContract.verify() called");

        requireThat(trx.getCommands().size() == 1, REQUIRE_SINGLE_COMMAND);
        final GameProposalCommand command = getSingleCommand(trx, GameProposalCommand.class);

        switch (command) {
            case CREATE:
                GameProposalCommand.validateCreateTrx(trx);
                break;

            case ACCEPT:
                GameProposalCommand.validateAcceptTrx(trx);
                break;

            case REJECT:
                GameProposalCommand.validateRejectTrx(trx);
                break;

            case CANCEL:
                GameProposalCommand.validateCancelTrx(trx);
                break;

            default:
                throw new CordaRuntimeException(UNKNOWN_COMMAND);
        }
    }

    static final String REQUIRE_SINGLE_COMMAND = "Require a single command";
    static final String UNKNOWN_COMMAND = "Unsupported command";
  }
