package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameProposalContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;
import net.corda.v5.ledger.utxo.ContractState;

@BelongsToContract(GameProposalContract.class)
public class GameProposalState implements ContractState, Counterparty {

    private final MemberX500Name issuer;
    private final MemberX500Name acquier;
    private final Piece.Color acquierColor;
    private final String message;
    private final UUID id;
    private final List<PublicKey> participants;

    @ConstructorForDeserialization
    public GameProposalState(
        MemberX500Name issuer,
        MemberX500Name acquier,
        Piece.Color acquierColor,
        String message,
        UUID id,
        List<PublicKey> participants
    ) {
        this.issuer = issuer;
        this.acquier = acquier;
        this.acquierColor = acquierColor;
        this.message = message;
        this.id = id;
        this.participants = participants;
    }

    public MemberX500Name getIssuer() {
        return issuer;
    }

    public MemberX500Name getAcquier() {
        return acquier;
    }

    public Piece.Color getAcquierColor() {
        return acquierColor;
    }

    public String getMessage() {
        return message;
    }

    public UUID getId() {
        return id;
    }

    public List<PublicKey> getParticipants() {
        return participants;
    }

    public MemberX500Name getWhitePlayerName() {
        return acquierColor == Piece.Color.WHITE ? getAcquier() : getIssuer();
    }

    public MemberX500Name getBlackPlayerName() {
        return acquierColor == Piece.Color.BLACK ? getAcquier() : getIssuer();
    }

    @Override
    public MemberX500Name getCounterpartyName(MemberX500Name myName) throws NotInvolved {
        if (issuer.compareTo(myName) == 0)
            return acquier;
    
        if (acquier.compareTo(myName) == 0)
            return issuer;
        
        throw new Counterparty.NotInvolved(myName, GameProposalState.class, this.id);
    }

}
