package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameResultContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;
import net.corda.v5.ledger.utxo.ContractState;

@BelongsToContract(GameResultContract.class)
public class GameResultState implements ContractState, Counterparty {

    private final MemberX500Name whitePlayerName;
    private final MemberX500Name blackPlayerName;

    private final Piece.Color victoryColor;

    private final UUID id;
    private final List<PublicKey> participants;
 
    @ConstructorForDeserialization
    public GameResultState(MemberX500Name whitePlayerName, MemberX500Name blackPlayerName, Piece.Color victoryColor, 
            UUID id, List<PublicKey> participants) {
        this.whitePlayerName = whitePlayerName;
        this.blackPlayerName = blackPlayerName;
        this.victoryColor = victoryColor;
        this.id = id;
        this.participants = participants;
    }

    public GameResultState(GameBoardState stateGameBoard, Piece.Color victoryColor) {
        this.whitePlayerName = stateGameBoard.getWhitePlayerName();
        this.blackPlayerName = stateGameBoard.getBlackPlayerName();
        this.victoryColor = victoryColor;

        this.id = UUID.randomUUID();
        this.participants = stateGameBoard.getParticipants();
    }

    public MemberX500Name getWhitePlayerName() {
        return whitePlayerName;
    }

    public MemberX500Name getBlackPlayerName() {
        return blackPlayerName;
    }

    public Piece.Color getVictoryColor() {
        return victoryColor;
    }

    public UUID getId() {
        return id;
    }

    @Override
    public List<PublicKey> getParticipants() {
        return participants;
    }

    @Override
    public MemberX500Name getCounterpartyName(MemberX500Name myName) throws NotInvolved {
        if (whitePlayerName.compareTo(myName) == 0)
            return blackPlayerName;
    
        if (blackPlayerName.compareTo(myName) == 0)
            return whitePlayerName;
        
        throw new Counterparty.NotInvolved(myName, GameResultState.class, this.id);
    }

}
