package djmil.cordacheckers.gameboard;

import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.FlowResult;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.Counterparty.NotInvolved;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class ListFlow implements ClientStartableFlow {

    private final static Logger log = LoggerFactory.getLogger(ListFlow.class);

    @CordaInject
    public MemberLookup memberLookup;

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @CordaInject
    public JsonMarshallingService jsonMarshallingService;

    @Suspendable
    @Override
    public String call(ClientRequestBody requestBody) {

        try {
            final var utxoGameBoardList = utxoLedgerService
                .findUnconsumedStatesByType(GameBoardState.class);
                // NOTE: lambda operations (aka .map) can not be used with UtxoLedgerService instances

            final List<GameBoardView> res = prepareGameBoardViewList(utxoGameBoardList);

            return new FlowResult(res)
                .toJsonEncodedString(jsonMarshallingService);
        }
        catch (Exception e) {
            log.warn("GameBoard.ListFlow failed to process utxo request body " + requestBody + " because: " + e.getMessage());
            return new FlowResult(e).toJsonEncodedString(jsonMarshallingService);
        }
    }

    @Suspendable
    private List<GameBoardView> prepareGameBoardViewList(List<StateAndRef<GameBoardState>> utxoGamaBoardList) {
        final MemberX500Name myName = memberLookup.myInfo().getName();
        List<GameBoardView> res = new LinkedList<GameBoardView>();

        for (StateAndRef<GameBoardState> sarGameBoard :utxoGamaBoardList) {
            try {
                res.add(prepareGameBoardView(sarGameBoard, myName));
            } catch (NotInvolved e) {
               log.warn(e.getMessage());
            }
        }

        return res;
    }

    @Suspendable
    private GameBoardView prepareGameBoardView(StateAndRef<GameBoardState> sarGameBoard, MemberX500Name myName) throws NotInvolved {
        final SecureHash trxId = sarGameBoard.getRef().getTransactionId();

        final UtxoLedgerTransaction utxoGameBoard = utxoLedgerService
            .findLedgerTransaction(trxId);

        return new GameBoardView(utxoGameBoard, myName);
    }
    
}
