package djmil.cordacheckers.user;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.naming.InvalidNameException;

import org.springframework.stereotype.Service;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;

@Service
public class HoldingIdentityResolver {
    static final Locale locale = Locale.getDefault();

    /*
     * NOTE: Proof-of-Concept impementation 
     *  Ideally, we want to be able to force update cache, had the corda
     *  cluster configuration changed (aka in case of a cache miss)
     */ 
    final Map<String, HoldingIdentity> cache; // PlayerName to ShortHash

    HoldingIdentityResolver(CordaClient cordaClient) {
        this.cache = setCache(cordaClient);
    }

    private static Map<String, HoldingIdentity> setCache(CordaClient cordaClient) {
        Map<String, HoldingIdentity> map = new HashMap<>();

        try {
            List<VirtualNode> vNodeList = cordaClient.getVirtualNodeList();

            for (VirtualNode vNode : vNodeList) {
                var identity = vNode.holdingIdentity();

                if (identity.isPlayer()) {
                    map.put(
                        identity.getName().toLowerCase(locale), 
                        identity
                        );
                }
            }
        } catch (InvalidNameException e) {
            // TODO: logs
            System.out.println("Unable to get ShorHash list of Corda VirtualNodes: "+e.getMessage());
        }

        return map;
    }

    /*
     * @param playerName 
     *  HoldingIdentity x500 name typically looks like 
     *  "CN=Bob, OU=Player, O=Checkers, L=Kviv, C=UA"
     *  CN - is a common name, expected to be unique for CordaCheckers setup.
     * 
     * @return HoldingIdentity
     */
    public HoldingIdentity getByUsername(String userName) {
        return this.cache.get(userName.toLowerCase(locale));
    }
}
